package com.starbuds.server.service.data.impl;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.pojo.db.DataAreaDO;
import com.starbuds.server.common.pojo.user.AreaDTO;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.data.dao.DataAreaDAO;
import com.starbuds.server.service.data.entity.AmapIpResult;
import com.starbuds.server.service.data.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static com.starbuds.server.service.data.util.SpringContextUtil.isDev;

/**
 * 区域服务实现
 *
 * @author Lay
 * @date 2019/12/25
 */
@Service(version = "1.0.0")
@Slf4j
public class AreaServiceImpl implements AreaService {
    @Autowired
    private DataAreaDAO dataAreaDAO;

    @Value("${amap.key}")
    private String amapKey;

    private final String GAODE_BASE_URL = "https://restapi.amap.com/";

    private String GET_IP_LOCATION_URL;

    /**
     * 逆地理编码
     */
    private String GET_GEOCODE_REGEO_URL;

    @PostConstruct
    private void init() {
        GET_IP_LOCATION_URL = String.format("%s%s?key=%s", GAODE_BASE_URL, "v3/ip", amapKey);
        GET_GEOCODE_REGEO_URL = String.format("%s%s?key=%s", GAODE_BASE_URL, "v3/geocode/regeo", amapKey);
    }

    /**
     * 获取区域列表
     *
     * @param parentId
     * @return
     */
    @Override
    public List<DataAreaDO> getAreaList(int parentId) {
        return dataAreaDAO.findByParentId(parentId);
    }

    @Override
    public DataAreaDO getAreaById(int areaId) {
        return dataAreaDAO.findByAreaId(areaId);
    }

    /**
     * 判断区域是否存在
     *
     * @param areaId
     * @return
     */
    @Override
    public boolean isAreaExist(int areaId) {
        return dataAreaDAO.countExist(areaId) > 0;
    }

    /**
     * 根据 ip 地址获取位置
     *
     * @param ip
     * @return
     */
    @Override
    public IpLocation getIpLocation(String ip) {
        String requestUrl = String.format("%s&ip=%s", GET_IP_LOCATION_URL, ip);
        try {
            String result = HttpUtil.get(requestUrl);
            if (isDev()) {
                log.info("高德地图获取地理位置 {} {}", ip, LogUtil.jsonObjLog(result));
            }
            if (StringUtils.isNotBlank(result)) {
                AmapIpResult ipResult = JSON.parseObject(result, AmapIpResult.class);
                if (ipResult != null && ipResult.getStatus().equals("1")) {
                    IpLocation ipLocation = new IpLocation();
                    ipLocation.setIp(ip);
                    ipLocation.setAreaId(Convert.toInt(ipResult.getAdcode()));
                    ipLocation.setProvince(ipResult.getProvince());
                    ipLocation.setCity(ipResult.getCity());

                    // 经纬度区间处理
                    String rectangle = ipResult.getRectangle();
                    String[] loc = rectangle.split(";");
                    if (loc.length > 1) {
                        String[] lbLoc = loc[0].split(",");
                        String[] rtLoc = loc[1].split(",");

                        // 左下坐标
                        ipLocation.setLbLongitude(Convert.toDouble(lbLoc[0], 0d));
                        ipLocation.setLbLatitude(Convert.toDouble(lbLoc[1], 0d));

                        // 右上坐标
                        ipLocation.setRtLongitude(Convert.toDouble(rtLoc[0], 0d));
                        ipLocation.setRtLatitude(Convert.toDouble(rtLoc[1], 0d));

                        // 查询中心坐标
                        DataAreaDO dataAreaDO = dataAreaDAO.selectLocationById(ipLocation.getAreaId());
                        if (dataAreaDO != null) {
                            ipLocation.setCenterLatitude(dataAreaDO.getAreaLatitude());
                            ipLocation.setCenterLongitude(dataAreaDO.getAreaLongitude());
                        }

                        return ipLocation;
                    }
                }
            }
        } catch (Exception e) {
            log.error("高德地图获取位置失败", e);
        }
        return null;
    }

    @Override
    public List<DataAreaDO> getAreaByAreaIds(Set<Integer> areaIds) {
        if (CollectionUtils.isEmpty(areaIds)) {
            return new ArrayList<>();
        }
        return dataAreaDAO.selectByAreaIds(areaIds);
    }

    @Override
    public List<AreaDTO> getAreasByAreaIds(Set<Integer> areaIds) {
        if (CollectionUtils.isEmpty(areaIds)) {
            return new ArrayList<>();
        }
        return dataAreaDAO.selectAreasByAreaIds(areaIds);
    }

    /**
     * 逆地理编码
     *
     * @param longitude
     * @param latitude
     * @return 城市名称
     */
    @Override
    public String getLocationCity(Double longitude, Double latitude) {
        // 经度在前，纬度在后，经纬度间以“,”分割，经纬度小数点后不要超过 6 位
        String requestUrl = String.format("%s&location=%s,%s", GET_GEOCODE_REGEO_URL, longitude, latitude);
        String result = null;
        try {
            result = HttpUtil.get(requestUrl);
        } catch (Exception e) {
            log.error("高德地图逆地理编码失败 " + requestUrl, e);
        }
        if (StringUtils.isBlank(result)) {
            log.error("高德地图逆地理编码失败 {}", requestUrl);
            return null;
        }
        if (!JSONObject.isValidObject(result)) {
            log.error("高德地图逆地理编码失败 {} 返回结果不是JSON对象 {}", requestUrl, result);
            return null;
        }

        JSONObject json = JSONObject.parseObject(result);
        JSONObject regeocode = json.getJSONObject("regeocode");
        if (regeocode == null) {
            log.error("高德地图逆地理编码失败 {} 返回结果regeocode为null {}", requestUrl, result);
            return null;
        }

        JSONObject addressComponent = regeocode.getJSONObject("addressComponent");
        if (addressComponent == null) {
            log.error("高德地图逆地理编码失败 {} 返回结果addressComponent为null {}", requestUrl, result);
            return null;
        }

        String city = addressComponent.getString("city");
        String province = addressComponent.getString("province");
        String district = addressComponent.getString("district");
        String country = addressComponent.getString("country");

        return StringUtils.isBlank(city) ? null : String.format("%s-%s-%s-%s", country, province, city, district);
    }
}
