package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.CoinGoodsCategoryVO;
import com.starbuds.server.common.pojo.data.CoinGoodsListItemVO;
import com.starbuds.server.common.pojo.data.CoinShopAnimationVO;
import com.starbuds.server.common.pojo.db.CoinGoodsDO;
import com.starbuds.server.common.pojo.db.PropsDO;
import com.starbuds.server.common.service.data.CoinGoodsService;
import com.starbuds.server.service.data.dao.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.common.constant.data.PropsTypeEnum.HEADWEAR_CARD;
import static com.starbuds.server.common.constant.data.PropsTypeEnum.MOUNTS_CARD;

/**
 * 积分商品服务实现
 *
 * @author Lay
 * @date 2020/3/30
 */
@Service(version = "1.0.0")
@Slf4j
public class CoinGoodsServiceImpl implements CoinGoodsService {

    @Resource
    CoinGoodsDAO coinGoodsDAO;

    @Resource
    CoinGoodsCategoryDAO coinGoodsCategoryDAO;

    @Resource
    private PropsDAO propsDAO;

    @Resource
    private MountsDAO mountsDAO;

    @Resource
    private HeadwearDAO headwearDAO;

    /**
     * 获取金币商品列表
     *
     * @param pageIndex
     * @param pageSize
     * @param keywords
     * @param categoryId
     * @param channelId
     * @return
     */
    @Override
    public PageList<CoinGoodsListItemVO> getCoinGoodsList(int pageIndex, int pageSize, String keywords, Long categoryId, Long channelId) {
        PageList<CoinGoodsListItemVO> pageList = new PageList<>(pageIndex, pageSize);
        List<CoinGoodsListItemVO> list = coinGoodsDAO.selectListItems(pageList.getPageCursor(), pageSize, categoryId);
        pageList.setTotal(coinGoodsDAO.countListItems(categoryId));
        if (CollectionUtil.isNotEmpty(list)) {
            // 道具id
            Set<Long> propsIds = new HashSet<>();
            for (CoinGoodsListItemVO item : list) {
                item.setGoodsPriceTypeName(WalletTypeEnum.toEnum(item.getGoodsPriceType()).getName());
                if (item.getItemId() != null) {
                    propsIds.add(item.getItemId());
                }
            }

            // 获得礼物动效
            Map<Long, CoinShopAnimationVO> animationMap = getAnimationMap(propsIds);
            list.stream()
                    .forEach(item -> {
                        CoinShopAnimationVO animation = animationMap.get(item.getItemId());
                        if (animation != null) {
                            item.setAnimationType(animation.getAnimationType());
                            item.setAnimationFile(animation.getAnimationFile());
                        }
                    });

            pageList.setList(list);
        }
        return pageList;
    }

    /**
     * 查询单个列表商品
     *
     * @param goodsId
     * @return
     */
    @Override
    public CoinGoodsListItemVO getCoinGoodsListItem(Long goodsId) {
        return coinGoodsDAO.selectListItem(goodsId);
    }

    /**
     * 查询商品
     *
     * @param goodsId
     * @return
     */
    @Override
    public CoinGoodsDO getCoinGoods(Long goodsId) {
        return coinGoodsDAO.selectById(goodsId);
    }

    /**
     * 获取类目
     *
     * @return
     */
    @Override
    public List<CoinGoodsCategoryVO> getCategories() {
        List<CoinGoodsCategoryVO> list = coinGoodsCategoryDAO.selectAll();
        if (CollectionUtil.isNotEmpty(list)) {
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 获得礼物动效Map
     *
     * @param propsIds 道具id
     * @return 道具id:动效
     */
    private Map<Long, CoinShopAnimationVO> getAnimationMap(Set<Long> propsIds) {
        if (CollectionUtil.isEmpty(propsIds)) {
            return Collections.EMPTY_MAP;
        }

        List<PropsDO> propsList = propsDAO.selectTypePropsExtStringByIds(propsIds);
        if (CollectionUtil.isEmpty(propsList)) {
            return Collections.EMPTY_MAP;
        }

        Set<Long> mountsIds = new HashSet<>();
        Set<Long> headwearIds = new HashSet<>();
        propsList = propsList.stream()
                .filter(props -> props != null)
                .filter(props -> props.getPropsType() != null)
                .filter(props -> props.getPropsExtString() != null)
                .filter(props -> NumberUtil.isLong(props.getPropsExtString()))
                .collect(Collectors.toList());
        propsList.stream()
                .forEach(props -> {
                    PropsTypeEnum propsType = PropsTypeEnum.toEnum(props.getPropsType());
                    Long id = Long.parseLong(props.getPropsExtString());
                    if (MOUNTS_CARD == propsType) {
                        mountsIds.add(id);
                    }
                    if (HEADWEAR_CARD == propsType) {
                        headwearIds.add(id);
                    }
                });

        List<CoinShopAnimationVO> animationList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(mountsIds)) {
            animationList.addAll(mountsDAO.selectAnimationByIds(mountsIds));
        }
        if (CollectionUtil.isNotEmpty(headwearIds)) {
            animationList.addAll(headwearDAO.selectAnimationByIds(headwearIds));
        }
        if (CollectionUtil.isEmpty(animationList)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, CoinShopAnimationVO> map = new HashMap<>();
        animationList.stream()
                .filter(vo -> vo != null)
                .filter(vo -> vo.getItemId() != null)
                .forEach(vo -> map.put(vo.getItemId(), vo));

        Map<Long, CoinShopAnimationVO> resultMap = new HashMap<>();
        propsList.stream()
                .forEach(props -> {
                    CoinShopAnimationVO vo = map.get(Long.parseLong(props.getPropsExtString()));
                    if (vo == null) {
                        return;
                    }
                    vo.setItemId(props.getPropsId());
                    resultMap.put(props.getPropsId(), vo);
                });
        return resultMap;
    }
}
