package com.starbuds.server.service.data.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSONObject;
import com.qiniu.cdn.CdnManager;
import com.qiniu.cdn.CdnResult;
import com.qiniu.common.QiniuException;
import com.qiniu.http.Response;
import com.qiniu.storage.Configuration;
import com.qiniu.storage.UploadManager;
import com.qiniu.util.Auth;
import com.qiniu.util.StringMap;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.pojo.data.QiniuConfig;
import com.starbuds.server.common.service.data.FileService;
import com.starbuds.server.service.data.task.NetworkImageToAvatarTask;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Value;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

/**
 * 文件服务实现
 *
 * @author Lay
 * @date 2019/12/17
 */
@Service(version = "1.0.0")
@Slf4j
public class FileServiceImpl implements FileService {

    @Value("${qiniu.accessKey}")
    String qiniuAccessKey;

    @Value("${qiniu.secretKey}")
    String qiniuSecretKey;

    @Value("${qiniu.bucket.user}")
    String qiniuBucketUser;

    @Value("${qiniu.domain.user}")
    String qiniuDomainUser;

    @Value("${qiniu.bucket.file}")
    String qiniuBucketFile;

    @Value("${qiniu.domain.file}")
    String qiniuDomainFile;

    @Value("${qiniu.bucket.media}")
    String qiniuBucketMedia;

    @Value("${qiniu.domain.media}")
    String qiniuDomainMedia;

    @Value("${qiniu.bucket.vod}")
    String qiniuBucketVod;

    @Value("${qiniu.domain.vod}")
    String qiniuDomainVod;

    @Value("${qiniu.bucket.feed}")
    String qiniuBucketFeed;

    @Value("${qiniu.domain.feed}")
    String qiniuDomainFeed;

    /**
     * 获取七牛配置
     *
     * @return
     */
    @Override
    public QiniuConfig getQiniuConfig() {
        QiniuConfig qiniuConfig = new QiniuConfig();
        qiniuConfig.setAccessKey(qiniuAccessKey);
        qiniuConfig.setSecretKey(qiniuSecretKey);
        qiniuConfig.setBucketVod(qiniuBucketVod);
        qiniuConfig.setDomainVod(qiniuDomainVod);
        return qiniuConfig;
    }

    /**
     * 保存网络图片为用户头像
     *
     * @param url
     * @param userId
     * @return
     */
    @Override
    public String saveNetworkImageToAvatar(String url, Long userId) {

        if (StringUtils.isBlank(url) || userId == null) {
            return null;
        }

        // 先生成文件名和令牌
        String key = getUserAvatarKey(userId);
        String token = getUserUploadToken(key, 3600);

        // 异步线程处理下载和转存
        NetworkImageToAvatarTask task = new NetworkImageToAvatarTask(url, key, token);
        task.execute();

        return qiniuDomainUser + key;
    }

    /**
     * 获取用户空间上传令牌
     *
     * @param key
     * @param expireSeconds
     * @return
     */
    @Override
    public String getUserUploadToken(String key, long expireSeconds) {
        Auth auth = Auth.create(qiniuAccessKey, qiniuSecretKey);
        StringMap putPolicy = new StringMap();
        JSONObject bodyParams = new JSONObject();
        bodyParams.put("key", "$(key)");
        bodyParams.put("fileSize", "$(fsize)");
        bodyParams.put("mimeType", "$(mimeType)");
        bodyParams.put("width", "$(imageInfo.width)");
        bodyParams.put("height", "$(imageInfo.height)");
        bodyParams.put("url", qiniuDomainUser + "$(key)");
        putPolicy.put("returnBody", bodyParams.toJSONString());

        return auth.uploadToken(qiniuBucketUser, key, expireSeconds, putPolicy);
    }

    /**
     * 获取文件空间上传令牌
     *
     * @param key
     * @param expireSeconds
     * @return
     */
    @Override
    public String getFileUploadToken(String key, long expireSeconds) {
        Auth auth = Auth.create(qiniuAccessKey, qiniuSecretKey);
        StringMap putPolicy = getFilePolicy(key);

        return auth.uploadToken(qiniuBucketFile, key, expireSeconds, putPolicy);
    }

    /**
     * 获取媒体空间上传令牌
     *
     * @param key
     * @param expireSeconds
     * @return
     */
    @Override
    public String getMediaUploadToken(String key, long expireSeconds) {
        Auth auth = Auth.create(qiniuAccessKey, qiniuSecretKey);
        StringMap putPolicy = new StringMap();
        JSONObject bodyParams = new JSONObject();
        bodyParams.put("key", "$(key)");
        bodyParams.put("fileSize", "$(fsize)");
        bodyParams.put("mimeType", "$(mimeType)");
        bodyParams.put("width", "$(imageInfo.width)");
        bodyParams.put("height", "$(imageInfo.height)");
        bodyParams.put("url", qiniuDomainMedia + "$(key)");
        putPolicy.put("returnBody", bodyParams.toJSONString());

        return auth.uploadToken(qiniuBucketMedia, key, expireSeconds, putPolicy);
    }

    /**
     * 获取内容空间上传令牌
     *
     * @param key
     * @param expireSeconds
     * @return
     */
    @Override
    public String getFeedUploadToken(String key, long expireSeconds) {
        Auth auth = Auth.create(qiniuAccessKey, qiniuSecretKey);
        StringMap putPolicy = new StringMap();
        JSONObject bodyParams = new JSONObject();
        bodyParams.put("key", "$(key)");
        bodyParams.put("fileSize", "$(fsize)");
        bodyParams.put("mimeType", "$(mimeType)");
        bodyParams.put("width", "$(imageInfo.width)");
        bodyParams.put("height", "$(imageInfo.height)");
        bodyParams.put("url", qiniuDomainFeed + "$(key)");
        putPolicy.put("returnBody", bodyParams.toJSONString());

        return auth.uploadToken(qiniuBucketFeed, key, expireSeconds, putPolicy);
    }

    /**
     * 获取用户头像上传 Key
     *
     * @param userId
     * @return
     */
    @Override
    public String getUserAvatarKey(Long userId) {
        return String.format("avatar/%d/%d.png", userId, System.currentTimeMillis());
    }

    /**
     * 获取用户封面上传 key
     *
     * @param userId
     * @return
     */
    @Override
    public String getUserCoverKey(Long userId) {
        return String.format("cover/%d/%d.png", userId, System.currentTimeMillis());
    }

    /**
     * 获取认证图片上传 key
     *
     * @param userId
     * @return
     */
    @Override
    public String getUserAuthKey(Long userId) {
        return String.format("auth/%d/%s%d.png", userId, RandomUtil.randomString(4), System.currentTimeMillis());
    }

    /**
     * 获取礼物动效文件 key
     *
     * @param fileExt
     * @return
     */
    @Override
    public String getGiftAnimationFileKey(String fileExt) {
        return String.format("animation/gift/%s.%s", IdUtil.fastSimpleUUID(), fileExt);
    }

    /**
     * 获取礼物图标 key
     *
     * @return
     */
    @Override
    public String getGiftIconKey() {
        return String.format("gift/icon/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取动效缩略图 key
     *
     * @return
     */
    @Override
    public String getGiftAnimationThumbKey() {
        return String.format("gift/animation/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取礼物标签 Key
     *
     * @return
     */
    @Override
    public String getGiftTagKey() {
        return String.format("gift/tag/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取道具 Key
     *
     * @return
     */
    @Override
    public String getPropsIconKey() {
        return String.format("props/icon/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取坐骑 Key
     *
     * @return
     */
    @Override
    public String getMountsKey() {
        return String.format("mounts/preview/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取勋章 Key
     *
     * @param type
     * @return
     */
    @Override
    public String getMedalKey(String type) {
        return String.format("medal/%s/%s.png", type, IdUtil.fastSimpleUUID());
    }

    /**
     * 获取等级 Key
     *
     * @param type
     * @return
     */
    @Override
    public String getLevelKey(String type) {
        return String.format("level/%s/%s.png", type, IdUtil.fastSimpleUUID());
    }

    /**
     * 获取举报图 key
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    @Override
    public String getReportKey(Long userId, Long targetUserId) {
        String fileName = SecureUtil.md5(String.valueOf(targetUserId)) + IdUtil.fastSimpleUUID();
        return String.format("report/%d/%s.png", targetUserId, fileName);
    }

    /**
     * 获取意见反馈Key
     *
     * @param userId
     * @return
     */
    @Override
    public String getSuggestKey(Long userId) {
        return String.format("suggest/%d/%s.png", userId, IdUtil.fastSimpleUUID());
    }

    @Override
    public String getAppUploadKey(Integer clientType, String versionCode, Long versionNum, Long channelId) {
        ClientType client = ClientType.toEnum(clientType);
        String downloadUrl = "";
        switch (client) {
            case Android:
                if (channelId == 0L || channelId == 3L) {
                    downloadUrl = String.format("android/timi_%s_%d_%d.apk", versionCode, versionNum, System.currentTimeMillis());
                } else {
                    downloadUrl = String.format("android/timi_%s_%d_%d_%d.apk", versionCode, versionNum, channelId, System.currentTimeMillis());
                }
                break;
            case iOS:
                if (channelId == 0L || channelId == 201L) {
                    downloadUrl = String.format("ios/timi_%s_%d.ipa", versionCode, versionNum);
                } else {
                    downloadUrl = String.format("ios/timi_%s_%d_%d.ipa", versionCode, versionNum, channelId);
                }
                break;
            default:
                break;
        }
        return downloadUrl;
    }

    @Override
    public String getActivityIconKey(Integer activityType) {
        return String.format("activityIcon/%d/%s.png", activityType, IdUtil.fastSimpleUUID());
    }

    @Override
    public String getActivityNoticeIconKey() {
        return String.format("activityNoticeIcon/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getLiveTagIconKey(Long channelId) {
        return String.format("liveTag/%d/%s.png", channelId, IdUtil.fastSimpleUUID());
    }

    @Override
    public String getBannerKey() {
        return String.format("banner/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getActivityBannerKey() {
        return String.format("activityBanner/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getVirtualCoinIconKey() {
        return String.format("coinIcon/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getSingleChatVideoUploadKey(String fileExtension) {
        return String.format("singleChatVideo/%s.%s", IdUtil.fastSimpleUUID(), fileExtension);
    }

    @Override
    public String getGoodsCategoryIconKey() {
        return String.format("goodsCategoryIcon/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getGoodsPicKey() {
        return String.format("goodsPic/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getAppIconUploadKey() {
        return String.format("appIcon/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getThirdPartyAppIconUploadKey() {
        return String.format("thirdPartyAppIcon/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getPosterImgUploadKey() {
        return String.format("posterImg/%s.png", IdUtil.fastSimpleUUID());
    }

    @Override
    public String getUserRechargeDailyExcelKey(String queryJson) {
        // 财务/充值/每日
        return String.format("finance/recharge/daily/%s", queryJson);
    }

    @Override
    public String uploadFile(byte[] fileByte, String key) {
        Configuration cfg = new Configuration();
        UploadManager uploadManager = new UploadManager(cfg);
        String token = getFileUploadToken(key, 300);

        InputStream inputStream = new ByteArrayInputStream(fileByte);
        try {
            Response response = uploadManager.put(inputStream, key, token, null, null);
            //解析上传成功的结果
            String res = response.bodyString();
            if (StringUtils.isBlank(res)) {
                log.error("七牛上传失败！返回数据为空，token={}", token);
                return null;
            }
            JSONObject result = JSONObject.parseObject(res);
            if (result == null) {
                log.error("七牛上传失败！返回数据格式错误！token={} res={}", token, res);
                return null;
            }
            String url = result.getString("url");
            if (StringUtils.isBlank(url)) {
                log.error("七牛上传失败！返回数据格式错误！token={} res={}", token, res);
                return null;
            }

            log.info("七牛上传成功！token={} res={}", token, res);
            return url;
        } catch (QiniuException ex) {
            Response r = ex.response;
            log.error("七牛上传失败！QiniuException=" + r.toString(), ex);
            try {
                log.error("七牛上传失败！QiniuException={}", r.bodyString());
            } catch (QiniuException ex2) {
                //ignore
            }
        }

        return null;
    }

    @Override
    public String getFeedImageKey(Long userId) {
        return String.format("image/%d/%d.png", userId, System.currentTimeMillis());
    }

    @Override
    public String getFeedVideoKey(Long userId) {
        return String.format("video/%d/%d.mp4", userId, System.currentTimeMillis());
    }

    @Override
    public String getFeedVideoCoverKey(Long userId) {
        return String.format("videoCover/%d/%d.png", userId, System.currentTimeMillis());
    }

    @Override
    public String getRtcRoomBackgroundKey() {
        return String.format("rtc/background/%d.png", System.currentTimeMillis());
    }

    @Override
    public String getRtcRoomCoverKey() {
        return String.format("rtc/roomCover/%d.png", System.currentTimeMillis());
    }

    @Override
    public String getBankLogoUploadConfig() {
        return String.format("bank/logo/%d.png", System.currentTimeMillis());
    }

    /**
     * 获取音乐文件上传key
     *
     * @param userId
     * @return
     */
    @Override
    public String getMusicUploadKey(Long userId) {
        return String.format("music/%d/%d.mp3", userId, System.currentTimeMillis());
    }

    /**
     * 获取通用文件上传配置
     *
     * @param fileExt
     * @return
     */
    @Override
    public String getCommonFileUploadKey(String fileExt) {
        return String.format("common/%s.%s", IdUtil.fastSimpleUUID(), fileExt);
    }

    /**
     * 获取头饰上传key
     *
     * @return
     */
    @Override
    public String getHeadwearKey() {
        return String.format("headwear/preview/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取技能icon上传key
     *
     * @return
     */
    @Override
    public String getSkillIconUploadKey() {
        return String.format("skillIcon/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取技能图片上传key
     *
     * @return
     */
    @Override
    public String getSkillImageUploadKey() {
        return String.format("skillImage/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取语音房分类背景图片上传key
     *
     * @return
     */
    @Override
    public String getRtcCategoryBackgroundImageUploadKey() {
        return String.format("rtc/category/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取技能形象照上传key
     *
     * @return
     */
    @Override
    public String getSkillImagePhotoUploadKey() {
        return String.format("skillImagePhoto/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取技能语音上传key
     *
     * @return
     */
    @Override
    public String getSkillVoiceUploadKey() {
        return String.format("skillVoice/%s.mp3", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取模块分类icon上传key
     *
     * @return
     */
    @Override
    public String getModuleIconUploadKey() {
        return String.format("module/icon/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 获取聊天气泡上传key
     *
     * @return
     */
    @Override
    public String getBubbleImageUploadKey() {
        return String.format("bubble/%s.png", IdUtil.fastSimpleUUID());
    }

    /**
     * 刷新 CDN 缓存
     *
     * @param url
     * @return
     */
    @Override
    public boolean refreshCDN(String url) {
        if (StringUtils.isBlank(url)) {
            return false;
        }

        Auth auth = Auth.create(qiniuAccessKey, qiniuSecretKey);
        CdnManager c = new CdnManager(auth);
        String[] urls = new String[]{
                url
        };
        try {
            //单次方法调用刷新的链接不可以超过100个
            CdnResult.RefreshResult result = c.refreshUrls(urls);
            System.out.println(result.code);
            if (result.code == 200) {
                return true;
            }
            throw new RuntimeException(String.valueOf(result.code));
        } catch (Exception e) {
            log.error("七牛刷新CDN缓存失败:", e);
        }
        return false;
    }

    private StringMap getFilePolicy() {
        return getFilePolicy(null);
    }

    private StringMap getFilePolicy(String key) {
        StringMap putPolicy = new StringMap();
        JSONObject bodyParams = new JSONObject();
        bodyParams.put("key", "$(key)");
        bodyParams.put("fileSize", "$(fsize)");
        bodyParams.put("mimeType", "$(mimeType)");
        bodyParams.put("width", "$(imageInfo.width)");
        bodyParams.put("height", "$(imageInfo.height)");
        bodyParams.put("url", qiniuDomainFile + "$(key)");
        putPolicy.put("returnBody", bodyParams.toJSONString());
        if (StringUtils.isNotBlank(key)) {
            putPolicy.put("scope", String.format("%s:%s", qiniuBucketFile, key));
        }

        return putPolicy;
    }
}
