package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.user.SwitchStatusEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.db.UserSwitchDO;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.data.dao.HeadwearDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 头饰服务实现
 *
 * @author Lay
 * @date 2020/8/28
 */
@Slf4j
@Service(version = "1.0.0")
public class HeadwearServiceImpl implements HeadwearService {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Resource
    HeadwearDAO headwearDAO;

    /**
     * 获取头饰
     *
     * @param headwearId
     * @return
     */
    @Override
    public HeadwearVO getHeadwear(Long headwearId) {
        if (headwearId != null) {
            return headwearDAO.selectVOById(headwearId);
        }
        return null;
    }

    /**
     * 获取头饰
     *
     * @param headwearIds
     * @return
     */
    @Override
    public Map<Long, HeadwearVO> getHeadwearMap(Set<Long> headwearIds) {
        if (CollectionUtil.isEmpty(headwearIds)) {
            return Collections.EMPTY_MAP;
        }
        return headwearDAO.selectByIds(headwearIds);
    }

    /**
     * 获取用户启用的头饰
     *
     * @param userId
     * @return
     */
    @Override
    public HeadwearVO getUserEnabledHeadwear(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.HEADWEAR);
        if (userSwitch != null) {
            if (SwitchStatusEnum.OPEN.getCode() == userSwitch.getSwitchStatus()) {
                Long headwearId = userSwitch.getItemId();
                if (headwearId != null) {
                    // 判断头饰是否过期
                    if (bagService.getItemQuantity(userId, BagItemTypeEnum.HEADWEAR, headwearId) > 0) {
                        return getHeadwear(headwearId);
                    }
                }
            }
        }
        return null;
    }

    /**
     * 获取用户启用的头饰id
     *
     * @param userId
     * @return
     */
    @Override
    public Long getUserEnabledHeadwearId(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.HEADWEAR);
        if (userSwitch != null) {
            if (SwitchStatusEnum.OPEN.getCode() == userSwitch.getSwitchStatus()) {
                Long headwearId = userSwitch.getItemId();
                if (headwearId != null) {
                    // 判断头饰是否过期
                    if (bagService.getItemQuantity(userId, BagItemTypeEnum.HEADWEAR, headwearId) > 0) {
                        return headwearId;
                    }
                }
            }
        }
        return null;
    }

    /**
     * 获取用户启用的头饰动效id
     *
     * @param userId
     * @return
     */
    @Override
    public Long getUserEnabledHeadwearAnimationId(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.HEADWEAR);
        log.info("userId {} , userSwitch {} ", userId, JSON.toJSONString(userSwitch));
        if (userSwitch != null) {
            if (SwitchStatusEnum.OPEN.getCode() == userSwitch.getSwitchStatus()) {
                Long headwearId = userSwitch.getItemId();
                log.info("userId {} , headwearId {} ", userId, headwearId);
                if (headwearId != null) {
                    // 判断头饰是否过期
                    if (bagService.getItemQuantity(userId, BagItemTypeEnum.HEADWEAR, headwearId) > 0) {
                        HeadwearVO headwear = getHeadwear(headwearId);
                        log.info("userId {} , headwear {} ", userId, JSON.toJSONString(headwear));
                        return Optional.ofNullable(headwear).map(HeadwearVO::getAnimationId).orElse(null);
                    }
                }
            }
        }
        return null;
    }

    /**
     * 获取用户启用的头饰
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, HeadwearVO> getUsersEnabledHeadwear(List<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        // 开关
        Map<Long, UserSwitchDO> switchMap = userService.getSwitchMap(new HashSet<>(userIds), SwitchTypeEnum.HEADWEAR);
        if (CollectionUtil.isEmpty(switchMap)) {
            return Collections.EMPTY_MAP;
        }
        // 用户id=>头饰id
        Map<Long, Long> userHeadwearMap = new HashMap<>();
        for (Long userId : userIds) {
            UserSwitchDO switchDO = switchMap.get(userId);
            if (switchDO != null && SwitchStatusEnum.OPEN.getCode() == switchDO.getSwitchStatus()) {
                Long headwearId = switchDO.getItemId();
                if (headwearId != null) {
                    userHeadwearMap.put(userId, headwearId);
                }
            }
        }
        if (CollectionUtil.isEmpty(userHeadwearMap)) {
            return Collections.EMPTY_MAP;
        }
        // 头饰id
        Set<Long> headwearIds = userHeadwearMap.values().stream().collect(Collectors.toSet());
        // 头饰
        Map<Long, HeadwearVO> headwearMap = getHeadwearMap(headwearIds);
        if (CollectionUtil.isEmpty(headwearMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, HeadwearVO> resultMap = new HashMap<>();
        for (Long userId : userIds) {
            resultMap.put(userId, headwearMap.get(userHeadwearMap.get(userId)));
        }
        return resultMap;
    }

    /**
     * 判断坐骑是否存在
     *
     * @param headwearId
     * @return
     */
    @Override
    public boolean isExists(Long headwearId) {
        return headwearDAO.countById(headwearId) > 0;
    }

    /**
     * 检查状态
     *
     * @param userId
     * @return
     */
    @Override
    public boolean checkEnabledHeadwearStatus(Long userId) {
        UserSwitchDO userSwitch = userService.getSwitch(userId, SwitchTypeEnum.HEADWEAR);
        if (userSwitch != null) {
            if (SwitchStatusEnum.OPEN.getCode() == userSwitch.getSwitchStatus()) {
                Long headwearId = userSwitch.getItemId();
                if (headwearId != null) {
                    // 判断头饰是否过期 过期了关闭头饰
                    if (bagService.getItemQuantity(userId, BagItemTypeEnum.HEADWEAR, headwearId) == 0) {
                        userSwitch.setSwitchStatus(SwitchStatusEnum.CLOSE.getCode());
                        userService.setSwitch(userSwitch);
                    }
                }
            }
        }

        return true;
    }
}
