package com.starbuds.server.service.data.impl;

import com.starbuds.server.common.annotation.i18n.I18nTag;
import com.starbuds.server.common.constant.i18n.I18nTypeEnum;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.*;
import com.starbuds.server.common.pojo.db.DataI18nDO;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.finance.PriceConfig;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.i18n.I18nService;
import com.starbuds.server.service.data.dao.DataI18nDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * 国际化服务接口实现类
 *
 * @author aoe
 * @date 2020/5/12
 */
@Slf4j
@Service(version = "1.0.0")
public class I18nServiceImpl implements I18nService {

    @Autowired
    private DataI18nDAO dataI18nDAO;

    @Override
    public int batchInsertOrUpdate(List<DataI18nDO> list) {
        return dataI18nDAO.batchInsertOrUpdate(list);
    }

    @Override
    public PageList<DataI18nVO> getPageList(int pageIndex, int pageSize) {
        PageList<DataI18nVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(dataI18nDAO.getPageList(page.getPageCursor(), pageSize));
        page.setTotal(dataI18nDAO.getPageListCount());
        return page;
    }

    @Override
    public <T> Object doI18n(Object obj, String lang) {
        if (obj == null) {
            return obj;
        }
        I18nTypeEnum langTypeEnum = I18nTypeEnum.toEnum(lang);
        if (I18nTypeEnum.EN != langTypeEnum) {
            // 目前只翻译英文
            return obj;
        }

        // 转换成统一格式
        List<Object> objList;
        if (obj instanceof List) {
            objList = (List) obj;
        } else {
            objList = new ArrayList<>(1);
            objList.add(obj);
        }

        doI18nByList(objList, langTypeEnum);
        return objList;
    }

    @Override
    public <T> Object doI18n(Object obj, String lang, int type) {
        List list = Collections.emptyList();
        switch (type) {
            case 1:
                list = getMedalList(obj);
                break;
            case 2:
                list = getMountsList(obj);
                break;
            case 3:
                list = getGiftList(obj);
                break;
            case 4:
                list = getPropsList(obj);
                break;
            case 5:
                list = getGoodsList(obj);
                break;
            case 6:
                getInitConfig(obj, lang);
                break;
            case 7:
                getCoinTypeNameMap(obj, lang);
                break;
        }

        doI18n(list, lang);

        return obj;
    }

    /**
     * 获取背包勋章列表
     *
     * @param obj
     * @return
     */
    private List<MedalVO> getMedalList(Object obj) {
        List<MedalVO> i18nList = new ArrayList<>();
        List<BagItem<MedalVO>> medalList = (List<BagItem<MedalVO>>) obj;
        if (medalList != null) {
            for (BagItem<MedalVO> bagItem : medalList) {
                if (bagItem == null) {
                    continue;
                }
                if (bagItem.getItem() != null) {
                    i18nList.add(bagItem.getItem());
                }
            }
        }
        return i18nList;
    }

    /**
     * 获取背包坐骑列表
     *
     * @param obj
     * @return
     */
    private List<MountsVO> getMountsList(Object obj) {
        List<BagItem<MountsVO>> list = (List<BagItem<MountsVO>>) obj;
        List<MountsVO> result = new ArrayList<>();
        if (list == null) {
            return result;
        }
        for (BagItem<MountsVO> bagItem : list) {
            if (bagItem == null) {
                continue;
            }
            if (bagItem.getItem() != null) {
                result.add(bagItem.getItem());
            }
        }

        return result;
    }

    /**
     * 获取背包礼物列表
     *
     * @param obj
     * @return
     */
    private List<GiftItemVO> getGiftList(Object obj) {
        List<BagItem<GiftItemVO>> list = (List<BagItem<GiftItemVO>>) obj;
        List<GiftItemVO> result = new ArrayList<>();
        if (list == null) {
            return result;
        }
        for (BagItem<GiftItemVO> bagItem : list) {
            if (bagItem == null) {
                continue;
            }
            if (bagItem.getItem() != null) {
                result.add(bagItem.getItem());
            }
        }

        return result;
    }

    /**
     * 获取背包道具列表
     *
     * @param obj
     * @return
     */
    private List<PropsVO> getPropsList(Object obj) {
        List<BagItem<PropsVO>> list = (List<BagItem<PropsVO>>) obj;
        List<PropsVO> result = new ArrayList<>();
        if (list == null) {
            return result;
        }
        for (BagItem<PropsVO> bagItem : list) {
            if (bagItem == null) {
                continue;
            }
            if (bagItem.getItem() != null) {
                result.add(bagItem.getItem());
            }
        }

        return result;
    }

    /**
     * 获取商品列表
     *
     * @param obj
     * @return
     */
    private List<CoinGoodsListItemVO> getGoodsList(Object obj) {
        PageList<CoinGoodsListItemVO> page = (PageList<CoinGoodsListItemVO>) obj;
        List<CoinGoodsListItemVO> result = new ArrayList<>();
        if (page == null) {
            return result;
        }
        List<CoinGoodsListItemVO> list = page.getList();
        if (list == null) {
            return result;
        }
        for (CoinGoodsListItemVO itemVO : list) {
            if (itemVO == null) {
                continue;
            }
            result.add(itemVO);
        }

        return result;
    }

    /**
     * 获取初始化配置
     *
     * @param obj
     * @return
     */
    private void getInitConfig(Object obj, String lang) {
        MutableResult mResult = (MutableResult) obj;
        if (mResult == null) {
            return;
        }

        ChannelInitConfig channelInitConfig = (ChannelInitConfig) mResult.get("channelConfig");
        List<DataForbiddenWordOptionVO> forbiddenWordOptions = (List<DataForbiddenWordOptionVO>) mResult.get("forbiddenWordOptions");

        if (channelInitConfig != null) {
            doI18n(channelInitConfig, lang);

            List<PayWayInitConfig> payWays = channelInitConfig.getPayWays();
            List<OauthInitConfig> oauths = channelInitConfig.getOauths();
            List<LiveCategory> liveCategories = channelInitConfig.getLiveCategories();
            List<SingleChatCategory> singleChatCategories = channelInitConfig.getSingleChatCategories();

            if (payWays != null) {
                doI18n(payWays, lang);
            }
            if (oauths != null) {
                doI18n(oauths, lang);
            }
            if (liveCategories != null) {
                doI18n(liveCategories, lang);
            }
            if (singleChatCategories != null) {
                doI18n(singleChatCategories, lang);
            }

            OneKeyLoginInitConfig oneKeyLogin = channelInitConfig.getOneKeyLogin();
            if (oneKeyLogin != null) {
                doI18n(oneKeyLogin, lang);
            }

        }
        if (forbiddenWordOptions != null) {
            doI18n(forbiddenWordOptions, lang);
        }
    }

    /**
     * 获取金币类型名称
     *
     * @param obj
     * @return
     */
    private void getCoinTypeNameMap(Object obj, String lang) {
        Map<Integer, String> map = (Map<Integer, String>) obj;
        if (map == null) {
            return;
        }

        // 顺序不变
        List<String> wordList = new ArrayList<>(map.size());
        for (Map.Entry<Integer, String> entry : map.entrySet()) {
            wordList.add(entry.getValue());
        }
        Map<String, DataI18nVO> i18nVOMap = dataI18nDAO.getListByIds(I18nTypeEnum.toEnum(lang).value(), wordList);
        if (i18nVOMap == null) {
            return;
        }
        for (Map.Entry<Integer, String> entry : map.entrySet()) {
            DataI18nVO i18nVO = i18nVOMap.get(entry.getValue());
            if (i18nVO == null || StringUtils.isBlank(i18nVO.getMsg())) {
                continue;
            }
            entry.setValue(i18nVO.getMsg());
        }
    }

    /**
     * 将对象中带有 I18nTag标签的属性 国际化
     *
     * @param objList      包含@I18nTag标签的对象列表
     * @param langTypeEnum 语言
     */
    private void doI18nByList(List<Object> objList, I18nTypeEnum langTypeEnum, Class... clazzArr) {
        if (objList == null || objList.isEmpty()) {
            return;
        }

        // 需要转国际化的字段列表
        List<Field> i18nFieldList = new ArrayList<>();
        // 需要转国际化的文字列表
        List<String> wordList = new ArrayList<>();
        // 提取国际化需要翻译的信息
        for (Object obj : objList) {
            if (obj == null) {
                continue;
            }
            collectByObj(i18nFieldList, wordList, obj);
        }
        // 翻译
        doTranslation(i18nFieldList, wordList, langTypeEnum, objList);
    }

    /**
     * 提取国际化需要翻译的信息
     *
     * @param i18nFieldList 需要转国际化的字段列表
     * @param wordList      需要转国际化的文字列表
     * @param obj
     */
    private void collectByObj(List<Field> i18nFieldList, List<String> wordList, Object obj, Class... clazzArr) {
        Field[] fields = obj.getClass().getDeclaredFields();
        for (Field field : fields) {
            // 获得所有的注解
            for (Annotation annotation : field.getDeclaredAnnotations()) {
                if (annotation instanceof I18nTag && "java.lang.String".equals(field.getType().getName())) {
                    field.setAccessible(true);
                    String word = null;
                    try {
                        word = (String) field.get(obj);
                        if (StringUtils.isBlank(word)) {
                            continue;
                        }
                    } catch (IllegalAccessException e) {
                        log.error("转国际化失败 " + obj + " 字段 " + field.getName(), e);
                    }
                    i18nFieldList.add(field);
                    wordList.add(word);
                }
            }
        }
    }

    /**
     * 翻译
     *
     * @param i18nFieldList
     * @param wordList
     * @param langTypeEnum
     * @param objList
     */
    private void doTranslation(List<Field> i18nFieldList, List<String> wordList, I18nTypeEnum langTypeEnum, List<Object> objList) {
        if (wordList.isEmpty()) {
            return;
        }

        Map<String, DataI18nVO> map = dataI18nDAO.getListByIds(langTypeEnum.value(), wordList);

        for (Object obj : objList) {
            for (Field field : i18nFieldList) {
                try {
                    String word = (String) field.get(obj);
                    DataI18nVO dataI18nVO = map.get(word);
                    if (dataI18nVO == null) {
                        continue;
                    }
                    String msg = dataI18nVO.getMsg();
                    if (StringUtils.isBlank(msg)) {
                        continue;
                    }

                    field.set(obj, msg);
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
