package com.starbuds.server.service.data.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.MusicStatusEnum;
import com.starbuds.server.common.pojo.api.RecordList;
import com.starbuds.server.common.pojo.data.MusicVO;
import com.starbuds.server.common.pojo.db.MusicDO;
import com.starbuds.server.common.pojo.db.UserMusicDO;
import com.starbuds.server.common.service.data.MusicSearchService;
import com.starbuds.server.common.service.data.MusicService;
import com.starbuds.server.service.data.dao.MusicDAO;
import com.starbuds.server.service.data.dao.UserMusicDAO;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/8/3 16:29
 * @Description 音乐
 */
@Service(version = "1.0.0")
public class MusicServiceImpl implements MusicService {

    @Resource
    MusicDAO musicDAO;

    @Resource
    UserMusicDAO userMusicDAO;

    @Resource
    MusicSearchService musicSearchService;

    /**
     * 添加音乐
     *
     * @param musicDO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addMusic(MusicDO musicDO) {
        int count = musicDAO.insert(musicDO);
        if (count > 0) {
            musicSearchService.saveOrUpdate(musicDO);
            return true;
        }
        return false;
    }

    /**
     * 获取上传列表
     *
     * @param userId
     * @param lastId
     * @param sceneType
     * @param pageSize
     * @return
     */
    @Override
    public RecordList<MusicVO, Long> getUploadList(Long userId, Long lastId, Integer sceneType, Integer pageSize) {
        RecordList<MusicVO, Long> recordList = new RecordList<>(pageSize);
        List<MusicVO> list = musicDAO.selectUploadList(userId, lastId, pageSize);
        if (CollectionUtil.isNotEmpty(list) && BoolIntVal.True != sceneType) {
            for (MusicVO vo : list) {
                vo.setMusicUrl(null);
            }
        }
        recordList.setList(list);
        recordList.setPageSize(pageSize);
        recordList.setLastId(CollectionUtils.isEmpty(list) ? null : list.get(list.size() - 1).getMusicId());
        return recordList;
    }

    /**
     * 获取创建者
     *
     * @param musicId
     * @return
     */
    @Override
    public Long getUserId(Long musicId) {
        return musicDAO.selectUserId(musicId);
    }

    /**
     * 判断音乐是否存在
     *
     * @param musicId
     * @return
     */
    @Override
    public boolean isMusicExist(Long musicId) {
        return musicDAO.countById(musicId) > 0;
    }

    /**
     * 判断音乐是否可下载
     *
     * @param musicId
     * @return
     */
    @Override
    public boolean isMusicDownloadable(Long musicId) {
        return musicDAO.countByIdAndStatus(musicId, MusicStatusEnum.USER_UPLOAD.getValue(), BoolIntVal.True) > 0;
    }

    /**
     * 删除音乐
     *
     * @param musicIds
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean delMusic(List<Long> musicIds) {
        if (CollectionUtil.isEmpty(musicIds)) {
            return true;
        }
        long curTime = System.currentTimeMillis();
        for (Long musicId : musicIds) {
            int count = musicDAO.updateStatus(musicId, MusicStatusEnum.USER_DEL.getValue(), curTime);
            if (count > 0) {
                MusicDO musicDO = new MusicDO();
                musicDO.setMusicId(musicId);
                musicDO.setStatus(MusicStatusEnum.USER_DEL.getValue());
                musicDO.setUpdateTime(curTime);
                musicSearchService.saveOrUpdate(musicDO);
            }
        }
        return true;
    }

    /**
     * 获取下载列表
     *
     * @param userId
     * @param lastId
     * @param pageSize
     * @return
     */
    @Override
    public RecordList<MusicVO, Long> getDownloadList(Long userId, Long lastId, int pageSize) {
        RecordList<MusicVO, Long> recordList = new RecordList<>(pageSize);
        List<MusicVO> list = userMusicDAO.selectDownloadList(userId, lastId, pageSize);
        recordList.setList(list);
        recordList.setPageSize(pageSize);
        recordList.setLastId(CollectionUtils.isEmpty(list) ? null : list.get(list.size() - 1).getMusicId());
        return recordList;
    }

    /**
     * 获取音乐url
     *
     * @param musicId
     * @return
     */
    @Override
    public String getMusicUrl(Long musicId) {
        return musicDAO.selectMusicUrl(musicId);
    }

    /**
     * 下载url
     *
     * @param musicId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean downloadMusic(Long musicId, Long userId) {
        long curTime = System.currentTimeMillis();
        int count = musicDAO.increaseDownloadCount(musicId, curTime);

        if (count > 0) {

            UserMusicDO userMusic = new UserMusicDO();
            userMusic.setUserId(userId);
            userMusic.setMusicId(musicId);
            userMusic.setDownloadCount(1);
            userMusic.setUpdateTime(curTime);
            userMusicDAO.insertOrUpdate(userMusic);

            MusicDO musicDO = new MusicDO();
            musicDO.setMusicId(musicId);
            musicDO.setUpdateTime(curTime);
            musicDO.setDownloadCount(musicDAO.selectDownloadCount(musicId) + 1);
            musicSearchService.saveOrUpdate(musicDO);

            return true;
        }
        return false;
    }

    /**
     * 获取音乐状态
     *
     * @param musicId
     * @return
     */
    @Override
    public Integer getMusicStatus(Long musicId) {
        return musicDAO.selectMusicStatus(musicId);
    }

    /**
     * 获取用户id
     *
     * @param musicIdList
     * @return
     */
    @Override
    public List<Long> getUserIds(List<Long> musicIdList) {
        if (CollectionUtil.isEmpty(musicIdList)) {
            return new ArrayList<>();
        }
        return musicDAO.selectUserIds(musicIdList);
    }

    /**
     * 获取上传数量
     *
     * @param userId
     * @return
     */
    @Override
    public int getUploadCount(Long userId) {
        return musicDAO.selectUploadCount(userId);
    }
}
