package com.starbuds.server.service.data.impl;

import cn.hutool.dfa.WordTree;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.data.WordFilterTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.db.WordFilterDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.user.UserPageListQO;
import com.starbuds.server.common.service.data.WordFilterService;
import com.starbuds.server.service.data.dao.WordFilterDAO;
import com.starbuds.server.service.data.es.dao.WordRepository;
import com.starbuds.server.service.data.es.entity.WordEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

import static com.starbuds.server.common.pojo.util.PageListUtil.nextPage;

/**
 * 单词过滤服务实现
 *
 * @author Lay
 * @date 2020/1/11
 */
@Slf4j
@Service(version = "1.0.0")
public class WordFilterServiceImpl implements WordFilterService {

    @Autowired
    WordFilterDAO wordFilterDAO;

    @Autowired
    WordRepository wordRepository;

    /**
     * 获取匹配的敏感词列表
     *
     * @param wordFilterType
     * @param content
     * @return
     */
    @Override
    public List<String> getMatchedPhrases(WordFilterTypeEnum wordFilterType, String content) {
        return getWordTree(wordFilterType).matchAll(content);
    }

    /**
     * 获取匹配的敏感词数量
     *
     * @param wordFilterType
     * @param content
     * @return
     */
    @Override
    public int getMatchedPhraseCount(WordFilterTypeEnum wordFilterType, String content) {
        return getMatchedPhrases(wordFilterType, content).size();
    }

    /**
     * 是否匹配到敏感词
     *
     * @param wordFilterType
     * @param content
     * @return
     */
    @Override
    public boolean isMatchedPhrases(WordFilterTypeEnum wordFilterType, String content) {
        return getWordTree(wordFilterType).isMatch(content);
    }

    /**
     * 替换敏感词
     *
     * @param wordFilterType
     * @param content
     * @param replaceTo
     * @return
     */
    @Override
    public String replaceMatchedPhrase(WordFilterTypeEnum wordFilterType, String content, String replaceTo) {
        List<String> matchedList = getWordTree(wordFilterType).matchAll(content);
        for (String matched : matchedList) {
            content = content.replaceAll(matched, replaceTo);
        }
        return content;
    }

    /**
     * 创建敏感词汇
     *
     * @param wordsDO 敏感词汇
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createWord(WordFilterDO wordsDO) {
        WordEntity wordEntity = new WordEntity();
        BeanUtils.copyProperties(wordsDO, wordEntity);
        log.info("同步敏感词 ---------- {}", wordEntity.getWordPhrase());
        wordRepository.save(wordEntity);
        return true;
    }

    /**
     * 编辑敏感词汇
     *
     * @param wordsDO 敏感词汇
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean editWord(WordFilterDO wordsDO) {
        WordEntity wordEntity = new WordEntity();
        BeanUtils.copyProperties(wordsDO, wordEntity);
        wordRepository.save(wordEntity);
        return true;
    }

    /**
     * 禁用词汇
     *
     * @param wordId 词汇id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean disableWord(Long wordId) {
        WordFilterDO wordFilterDO = wordFilterDAO.selectById(wordId);
        WordEntity wordEntity = new WordEntity();
        BeanUtils.copyProperties(wordFilterDO, wordEntity);
        wordEntity.setWordStatus(CommonStatus.Disabled);
        wordRepository.save(wordEntity);
        return true;
    }

    /**
     * 启用词汇
     *
     * @param wordId 词汇id
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean enableWord(Long wordId) {
        WordFilterDO wordFilterDO = wordFilterDAO.selectById(wordId);
        WordEntity wordEntity = new WordEntity();
        BeanUtils.copyProperties(wordFilterDO, wordEntity);
        wordEntity.setWordStatus(CommonStatus.Normal);
        wordRepository.save(wordEntity);
        return true;
    }

    /**
     * 删除词汇
     *
     * @param wordId 词汇id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteWord(Long wordId) {
        WordFilterDO wordFilterDO = wordFilterDAO.selectById(wordId);
        WordEntity wordEntity = new WordEntity();
        BeanUtils.copyProperties(wordFilterDO, wordEntity);
        wordEntity.setWordStatus(CommonStatus.Deleted);
        wordRepository.save(wordEntity);
        return true;
    }

    /**
     * 获取敏感词汇
     *
     * @param wordFilterType
     * @return
     */
    @Override
    public List<String> getWords(WordFilterTypeEnum wordFilterType) {
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        boolQueryBuilder.must(QueryBuilders.matchQuery("wordStatus", CommonStatus.Normal));
        BoolQueryBuilder shouldQueryBuilder = QueryBuilders.boolQuery();
        shouldQueryBuilder.should(QueryBuilders.matchQuery("filterType", 0));
        shouldQueryBuilder.should(QueryBuilders.matchQuery("filterType", wordFilterType.getValue()));
        shouldQueryBuilder.should(QueryBuilders.wildcardQuery("filterType", String.format("*%s*", wordFilterType.getValue())));
        boolQueryBuilder.must(shouldQueryBuilder);
        Iterable iterable = null;
        try {
            iterable = wordRepository.search(boolQueryBuilder);
        } catch (Exception e) {
            log.error("ES服务异常", e);
            return Collections.emptyList();
        }
        if (iterable == null) {
            return Collections.emptyList();
        }
        Iterator<WordEntity> iterator = iterable.iterator();
        List<String> words = new ArrayList<>();
        while (iterator.hasNext()) {
            WordEntity next = iterator.next();
            words.add(next.getWordPhrase());
        }
        return words;
    }

    @Override
    public int syncAll2Es() {
        log.info("同步敏感词开始");
        Integer pageIndex = 0;
        // PageList 中最大为 50
        int pageSize = 50;

        // 同步敏感词数量
        AtomicInteger count = new AtomicInteger();
        while (true) {
            UserPageListQO qo = new UserPageListQO();
            qo.setPageIndex(pageIndex);
            qo.setPageSize(pageSize);
            List<WordFilterDO> words = wordFilterDAO.getWordPageList(pageIndex, pageSize);
            if (CollectionUtils.isEmpty(words)) {
                break;
            }
            if (CollectionUtils.isEmpty(words)) {
                continue;
            }

            words.stream()
                    .filter(Objects::nonNull)
                    .filter(word -> word.getWordId() != null)
                    .filter(word -> StringUtils.isNotBlank(word.getWordPhrase()))
                    .forEach(word -> {
                        WordFilterDO wordFilterDO = new WordFilterDO();
                        BeanUtils.copyProperties(word, wordFilterDO);
                        boolean result = createWord(wordFilterDO);
                        if (result) {
                            count.getAndIncrement();
                        }
                    });

            pageIndex += pageSize;
        }
        log.info("同步敏感词到ES结束，共 {} 条", count);

        return count.get();
    }


    private WordTree getWordTree(WordFilterTypeEnum wordFilterType) {
        // ES 查词

        WordTree wordTree = new WordTree();
        wordTree.addWords(getWords(wordFilterType));

        return wordTree;
    }
}
