package com.starbuds.server.service.data.provider;

import com.starbuds.server.common.constant.data.DataGreenTypeEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.data.GreenCompanyTypeEnum;
import com.starbuds.server.common.pojo.api.CallApiTrace;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.DataGreenHisDO;
import com.starbuds.server.common.service.data.DataGreenHisService;
import com.starbuds.server.service.data.entity.green.GreenRobotAuditResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.BeanUtils;

import javax.annotation.Resource;
import java.util.function.Function;
import java.util.function.Supplier;

/**
 * 内容审核
 *
 * @author aoe
 * @date 2021/8/20
 */
@Slf4j
public abstract class AbstractGreenProvider {

    @Resource
    private DataGreenHisService dataGreenHisService;

    /**
     * 图片审核
     *
     * @param imageUrl 图片地址
     * @param bizId    审核业务id
     * @param bizType  审核业务类型
     * @return
     */
    public GreenAuditTrace imgAudit(String imageUrl, String uniqueID, Long bizId, GreenBizTypeEnum bizType, DataGreenTypeEnum dataGreenType) {
        return audit(bizId, bizType, dataGreenType, () -> imgCensor(imageUrl, uniqueID, bizType), (rsp) -> imgAuditResult(rsp));
    }

    /**
     * 文本审核
     *
     * @param text    文本内容
     * @param bizId   审核业务id
     * @param bizType 审核业务类型
     * @return
     */
    public GreenAuditTrace textAudit(String text, String uniqueID, Long bizId, GreenBizTypeEnum bizType, DataGreenTypeEnum dataGreenType) {
        return audit(bizId, bizType, dataGreenType, () -> textCensor(text, uniqueID, bizType), (rsp) -> textAuditResult(rsp));
    }

    /**
     * 图片审核
     *
     * @param imageUrl
     * @param uniqueID 唯一ID,可以是用户ID，只要唯一用于标记
     * @return
     */
    protected abstract CallApiTrace imgCensor(String imageUrl, String uniqueID, GreenBizTypeEnum bizType);

    /**
     * 文本审核
     *
     * @param text 文本内容
     * @return
     */
    protected abstract CallApiTrace textCensor(String text, String uniqueID, GreenBizTypeEnum bizType);

    /**
     * 获得图片审核结果
     *
     * @param rsp 图片审核返回数据
     * @return
     */
    protected abstract GreenRobotAuditResult imgAuditResult(String rsp);

    /**
     * 获得文本审核结果
     *
     * @param rsp 文本审核返回数据
     * @return
     */
    protected abstract GreenRobotAuditResult textAuditResult(String rsp);

    /**
     * 内容审核公司类型
     *
     * @return
     */
    public abstract GreenCompanyTypeEnum companyType();

    /**
     * 内容审核
     *
     * @param bizId
     * @param bizType
     * @param callApiTraceSupplier
     * @param robotAuditResultFunction
     * @return
     */
    private GreenAuditTrace audit(Long bizId, GreenBizTypeEnum bizType, DataGreenTypeEnum dataGreenType,
                                  Supplier<CallApiTrace> callApiTraceSupplier,
                                  Function<String, GreenRobotAuditResult> robotAuditResultFunction) {
        GreenAuditTrace trace = new GreenAuditTrace();
        trace.setBizId(bizId);
        trace.setBizType(bizType);
        trace.setDataType(dataGreenType);

        // 计算API耗时
        long startTime = System.currentTimeMillis();
        CallApiTrace callApiTrace = callApiTraceSupplier.get();
        long spendTime = System.currentTimeMillis() - startTime;
        BeanUtils.copyProperties(callApiTrace, trace);
        trace.setSpendTime(NumberUtils.toInt(spendTime + ""));

        // 解析审核结果
        GreenRobotAuditResult robotAuditResult = robotAuditResultFunction.apply(trace.getRspData());
        log.info("robotAuditResult.getRobotAuditStatus():" + robotAuditResult.getRobotAuditStatus());
        trace.setRobotAuditStatus(robotAuditResult.getRobotAuditStatus());
        BeanUtils.copyProperties(robotAuditResult, trace);

        DataGreenHisDO po = new DataGreenHisDO();
        BeanUtils.copyProperties(trace, po);
        po.setBizType(trace.getBizType().getCode());
        po.setDataType(trace.getDataType().getCode());
        po.setRobotAuditStatus(trace.getRobotAuditStatus().value());
        GreenCompanyTypeEnum companyType = companyType();
        companyType = companyType == null ? GreenCompanyTypeEnum.UNKNOWN : companyType;
        po.setCompanyType(companyType.getCode());
        dataGreenHisService.create(po);

        return trace;
    }
}
