package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.data.GreenCompanyTypeEnum;
import com.starbuds.server.common.pojo.api.CallApiTrace;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.data.config.BaiduGreenConfig;
import com.starbuds.server.service.data.entity.green.GreenRobotAuditResult;
import com.starbuds.server.service.data.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 百度内容审核
 * https://cloud.baidu.com/doc/ANTIPORN/s/skk9093a1
 *
 * @author aoe
 * @date 2021/8/20
 */
@Slf4j
@Component(value = "baiduGreenProvider")
public class BaiduGreenProvider extends AbstractGreenProvider {

    @Resource
    private BaiduGreenConfig config;

    private final String LOG_PREFIX = "百度内容审核-";

    @Reference(version = "1.0.0")
    private RedisService redisService;

    /**
     * 审核状态Map
     */
    private Map<Integer, GreenAuditStatusEnum> auditStatusMap;

    @PostConstruct
    private void init() {
        // 1.合规，2.不合规，3.疑似，4.审核失败
        auditStatusMap = new HashMap<>();
        auditStatusMap.put(1, GreenAuditStatusEnum.PASSED_ROBOT);
        auditStatusMap.put(2, GreenAuditStatusEnum.BLOCK_ROBOT);
        auditStatusMap.put(3, GreenAuditStatusEnum.REVIEW_ROBOT);
        auditStatusMap.put(4, GreenAuditStatusEnum.REVIEW_ROBOT);
    }

    /**
     * 图片审核
     *
     * @param imageUrl
     * @return
     */
    @Override
    protected CallApiTrace imgCensor(String imageUrl, String uniqueID, GreenBizTypeEnum bizType) {
        String url = config.imgCensorHost() + "?access_token=" + getAccessTokenByCache();
        Map<String, String> param = new HashMap<>();
        param.put("imgUrl", imageUrl);

        CallApiTrace trace = new CallApiTrace();
        trace.setReqUrl(url);
        trace.setReqData(JSON.toJSONString(param));
        trace.setData(imageUrl);

        String rsp = null;
        try {
            rsp = HttpUtil.postForm(url, param);
        } catch (IOException e) {
            log.error(LOG_PREFIX + "图片审核失败，IO 异常", e);
            trace.setErrMsg(e.getMessage());
        }

        trace.setRspData(rsp);
        return trace;
    }

    /**
     * 文本审核
     *
     * @param text 文本内容
     * @return
     */
    @Override
    protected CallApiTrace textCensor(String text, String uniqueID, GreenBizTypeEnum bizType) {
        String url = config.textCensorHost() + "?access_token=" + getAccessTokenByCache();
        Map<String, String> param = new HashMap<>();
        param.put("text", text);

        CallApiTrace trace = new CallApiTrace();
        trace.setReqUrl(url);
        trace.setReqData(JSON.toJSONString(param));
        trace.setData(text);

        String rsp = null;
        try {
            rsp = HttpUtil.postForm(url, param);
        } catch (IOException e) {
            log.error(LOG_PREFIX + "文本审核失败，IO 异常", e);
            trace.setErrMsg(e.getMessage());
        }

        trace.setRspData(rsp);
        return trace;
    }

    @Override
    protected GreenRobotAuditResult imgAuditResult(String rsp) {
        return parseRsp(rsp);
    }

    @Override
    protected GreenRobotAuditResult textAuditResult(String rsp) {
        return parseRsp(rsp);
    }

    @Override
    public GreenCompanyTypeEnum companyType() {
        return GreenCompanyTypeEnum.BAI_DU;
    }

    /**
     * 解析内容审核结果
     *
     * @param rsp
     * @return
     */
    private GreenRobotAuditResult parseRsp(String rsp) {
        GreenRobotAuditResult result = new GreenRobotAuditResult();
        JSONObject json = JSON.parseObject(rsp);
        if (json == null) {
            result.setRobotAuditStatus(GreenAuditStatusEnum.BLOCK_RSP_ERROR);
            result.setFailMsg("返回结果不是JSON格式 " + rsp);
            return result;
        }

        String statusKye = "conclusionType";
        result.setStatusKye(statusKye);

        int conclusionType = NumberUtils.toInt(json.getString(statusKye));
        if (!auditStatusMap.containsKey(conclusionType)) {
            result.setRobotAuditStatus(GreenAuditStatusEnum.BLOCK_RSP_ERROR);
            result.setFailMsg("conclusionType 异常 " + conclusionType);
            return result;
        }

        result.setRobotAuditStatus(auditStatusMap.get(conclusionType));
        if (conclusionType != 1) {
            result.setFailMsg(json.getString("conclusion"));

            // 尝试解析具体错误原因
            JSONArray data = json.getJSONArray("data");
            if (data != null) {
                JSONObject firstData = data.getJSONObject(0);
                if (firstData != null) {
                    String msg = firstData.getString("msg");
                    if (StringUtils.isNotBlank(msg)) {
                        result.setFailMsg(msg);
                    }
                }
            }
        }

        return result;
    }

    /**
     * 从缓存获取AccessToken
     *
     * @return
     */
    private String getAccessTokenByCache() {
        String key = "baidu:green:access:token:" + LocalDate.now().getDayOfYear();
        String token = redisService.get(key);
        if (StringUtils.isNotBlank(token)) {
            return token;
        }
        token = getAccessToken();
        if (StringUtils.isNotBlank(token)) {
            redisService.set(key, token, 25, TimeUnit.HOURS);
        }
        return token;
    }

    /**
     * 获取API访问token
     * 该token有一定的有效期，需要自行管理，当失效时需重新获取.
     * curl -i -k 'https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id=Za3r7GjbMyVWePZ40RGWpoN2&client_secret=SLHv45eZWwiCGBpVcwB5TIGLT6Nr9eth'
     *
     * @return null 获取失败
     * @throws IOException
     */
    private String getAccessToken() {
        String url = config.authHost()
                // 1. grant_type为固定参数
                + "?grant_type=client_credentials"
                // 2. 官网获取的 API Key
                + "&client_id=" + config.apiKey()
                // 3. 官网获取的 Secret Key
                + "&client_secret=" + config.secretKey();
        String res;
        try {
            res = HttpUtil.get(url);
        } catch (IOException e) {
            log.error(LOG_PREFIX + "获取API访问 token 失败，IO 异常", e);
            return null;
        }
        if (StringUtils.isBlank(res)) {
            log.error(LOG_PREFIX + "获取API访问 token 失败，返回结果为空");
            return null;
        }
        if (Boolean.FALSE.equals(JSON.isValid(res))) {
            log.error(LOG_PREFIX + "获取API访问 token 失败，返回结果不是JSON格式 res = " + res);
            return null;
        }
        JSONObject json = JSON.parseObject(res);
        return json.getString("access_token");
    }
}

