package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.DataGreenHisDO;
import com.starbuds.server.service.data.dao.DataGreenHisDAO;
import com.starbuds.server.common.pojo.data.qo.DataGreenHisPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.*;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * 内容审核记录
 */
@Component
@Slf4j
public class DataGreenHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private DataGreenHisDAO dataGreenHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public DataGreenHisDO create(DataGreenHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = dataGreenHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("内容审核记录创建成功 {}", json);
            return po;
        }
        log.warn("内容审核记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<DataGreenHisDO> list) {
        boolean flag = dataGreenHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(DataGreenHisDO po) {
        boolean flag = dataGreenHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(DataGreenHisDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = dataGreenHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public DataGreenHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return dataGreenHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return dataGreenHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<DataGreenHisDO> getPageList(DataGreenHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<DataGreenHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(dataGreenHisDAO.selectPageList(qw));
        page.setTotal(dataGreenHisDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, DataGreenHisDO> getMapByIds(Collection ids) {
        return dataGreenHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 根据审核业务id获取内容审核记录Map
     *
     * @param bizIds 审核业务id集合
     * @return
     */
    public Map<Long, DataGreenHisDO> getMapByBizIds(Collection bizIds) {
        if (CollectionUtils.isEmpty(bizIds)) {
            return Collections.emptyMap();
        }

        List<DataGreenHisDO> list = dataGreenHisDAO.selectByBizIds(bizIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }

        Map<Long, DataGreenHisDO> map = new HashMap();
        list.stream()
                .filter(Objects::nonNull)
                .filter(po -> po.getBizId() != null)
                // 如果有重复的bizId，覆盖
                .forEach(po -> map.put(po.getBizId(), po));
        return map;
    }
}