package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.data.OcrResultVO;
import com.starbuds.server.service.data.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Ocr识别-百度
 * 文档：https://cloud.baidu.com/doc/OCR/s/rk3h7xzck
 *
 * @author aoe
 * @date 2021/6/21
 */
@Slf4j
@Component
public class OcrBaiduProvider {

    @Value("${baidu.ocr.appId}")
    private String appId;

    @Value("${baidu.ocr.apiKey}")
    private String apiKey;

    @Value("${baidu.ocr.secretKey}")
    private String secretKey;

    @Value("${baidu.ocr.accessTokenUrl}")
    private String accessTokenUrl;

    @Value("${baidu.ocr.idCardUrl}")
    private String idCardUrl;

    @PostConstruct
    private void init() {
        accessTokenUrl = String.format("%s?grant_type=client_credentials&client_id=%s&client_secret=%s",
                accessTokenUrl, apiKey, secretKey);
        log.info("百度Ocr获取Access Token Url {}", accessTokenUrl);
    }

    /**
     * Ocr识别
     *
     * @param frontUrl 身份证正面照片url
     * @return
     */
    public OcrResultVO doOcr(String frontUrl) {
        log.info("开始调用doOcr方法");
        OcrResultVO vo = new OcrResultVO();
        vo.setSuccess(false);

        String accessToken = getAccessToken();
        if (StringUtils.isBlank(accessToken)) {
            vo.setErrMsg("获取Access Token失败");
            return vo;
        }

        String url = String.format("%s?access_token=%s", idCardUrl, accessToken);
        Map<String, String> params = new HashMap<>();
        params.put("url", frontUrl);
        // 身份证含照片的一面
        params.put("id_card_side", "front");
        String res;
        try {
            log.info("开始调用百度ocr接口");
            res = HttpUtil.postForm(url, params);
            log.info("百度OCR结果 url = {} params = {} res = {}", url, JSON.toJSONString(params), res);
        } catch (IOException e) {
            log.error("百度Ocr识别失败 url = " + url + " params = " + JSON.toJSONString(params), e);
            vo.setErrMsg("Ocr识别失败");
            return vo;
        }

        log.info("开始解析ocr数据image_status");
        vo.setResult(res);
        JSONObject json = JSON.parseObject(res);
        String imageStatus = json.getString("image_status");
        if (!"normal".equals(imageStatus)) {
            if ("reversed_side".equals(imageStatus)) {
                vo.setErrMsg("身份证正反面颠倒");
            } else if ("non_idcard".equals(imageStatus)) {
                vo.setErrMsg("上传的图片中不包含身份证");
            } else if ("blurred".equals(imageStatus)) {
                vo.setErrMsg("身份证模糊");
            } else if ("other_type_card".equals(imageStatus)) {
                vo.setErrMsg("其他类型证照");
            } else if ("over_exposure".equals(imageStatus)) {
                vo.setErrMsg("身份证关键字段反光或过曝");
            } else if ("over_dark".equals(imageStatus)) {
                vo.setErrMsg("身份证欠曝（亮度过低）");
            } else {
                vo.setErrMsg("未知错误");
            }
            return vo;
        }

        log.info("开始解析ocr数据idcard_number_type");
        if (!"1".equals(json.getString("idcard_number_type"))) {
            vo.setErrMsg("身份证信息有误");
        }
        JSONObject wordsResult = json.getJSONObject("words_result");
        if (wordsResult == null) {
            vo.setErrMsg("返回失败信息有误");
            return vo;
        }

        JSONObject numJSON = wordsResult.getJSONObject("公民身份号码");
        if (numJSON == null) {
            vo.setErrMsg("无法获取身份证号码");
            return vo;
        }
        String num = numJSON.getString("words");
        if (StringUtils.isBlank(num)) {
            vo.setErrMsg("返回的身份证号码有误");
            return vo;
        }
        vo.setNum(num);

        JSONObject nameJSON = wordsResult.getJSONObject("姓名");
        if (nameJSON == null) {
            vo.setErrMsg("无法获取身份证姓名");
            return vo;
        }
        String name = nameJSON.getString("words");
        if (StringUtils.isBlank(name)) {
            vo.setErrMsg("返回的身份证姓名有误");
            return vo;
        }
        vo.setName(name);

        vo.setSuccess(true);
        return vo;
    }

    /**
     * 获取Access Token
     *
     * @return
     */
    private String getAccessToken() {
        String res;
        try {
            res = HttpUtil.postForm(accessTokenUrl, null);
        } catch (IOException e) {
            log.error("百度Ocr获取Access Token 失败!", e);
            return null;
        }

        JSONObject json = JSON.parseObject(res);
        String accessTokenKey = "access_token";
        if (!json.containsKey(accessTokenKey)) {
            log.error("百度Ocr获取Access Token失败，结果中不包含 access_token！res = " + res);
            return null;
        }
        return json.getString(accessTokenKey);
    }
}
