package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.data.GreenCompanyTypeEnum;
import com.starbuds.server.common.constant.feed.FeedTypeEnum;
import com.starbuds.server.common.pojo.api.CallApiTrace;
import com.starbuds.server.service.data.config.ShumeiConfig;
import com.starbuds.server.service.data.entity.green.GreenRobotAuditResult;
import com.starbuds.server.service.data.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/***
 * @author qinjiang
 * @description 数美内容审核
 * @date 2021/9/1
 */
@Slf4j
@Component(value = "shumeiGreenProvider")
public class ShumeiGreenProvider extends AbstractGreenProvider {

    @Resource
    private ShumeiConfig config;

    private final String LOG_PREFIX = "数美内容审核-";

    /**
     * 审核状态Map
     */
    private Map<String, GreenAuditStatusEnum> auditStatusMap;

    @PostConstruct
    private void init() {
        auditStatusMap = new HashMap<>();
        auditStatusMap.put("PASS", GreenAuditStatusEnum.PASSED_ROBOT);//通过
        auditStatusMap.put("REVIEW", GreenAuditStatusEnum.REVIEW_ROBOT);//需要审核
        auditStatusMap.put("REJECT", GreenAuditStatusEnum.BLOCK_ROBOT);//拒绝
        auditStatusMap.put("VERIFY", GreenAuditStatusEnum.REVIEW_ROBOT);//需要验证
    }

    /**
     * 图片审核
     *
     * @param imageUrl
     * @return
     */
    @Override
    protected CallApiTrace imgCensor(String imageUrl, String uniqueID, GreenBizTypeEnum bizType) {
        return censorRequest("album", imageUrl, uniqueID, false, FeedTypeEnum.PHOTO);
    }

    /**
     * 文本审核
     *
     * @param text 文本内容
     * @return
     */
    @Override
    protected CallApiTrace textCensor(String text, String uniqueID, GreenBizTypeEnum bizType) {
        String eventType = "comment";
        if (bizType.equals(GreenBizTypeEnum.USER_NAME)) {
            eventType = "nickname";
        } else if (bizType.equals(GreenBizTypeEnum.USER_SIGN)) {
            eventType = "signature";
        } else if (bizType.equals(GreenBizTypeEnum.FEED_TEXT)) {
            eventType = "comment";
        } else if (bizType.equals(GreenBizTypeEnum.SINGLE_RTC_ROOM_NAME)) {
            eventType = "nickname";
        }

        return censorRequest(eventType, text, uniqueID, true, FeedTypeEnum.PHOTO);
    }

    /**
     * 获得图片审核结果
     *
     * @param rsp 图片审核返回数据
     * @return
     */
    @Override
    protected GreenRobotAuditResult imgAuditResult(String rsp) {
        return getResult(rsp);
    }

    /**
     * 获得文本审核结果
     *
     * @param rsp 文本审核返回数据
     * @return
     */
    @Override
    protected GreenRobotAuditResult textAuditResult(String rsp) {
        return getResult(rsp);
    }

    /**
     * 内容审核公司类型
     *
     * @return
     */
    @Override
    public GreenCompanyTypeEnum companyType() {
        return GreenCompanyTypeEnum.SHU_MEI;
    }

    /**
     * 处理审查请求数据
     *
     * @param eventType
     * @param eventValue
     * @param isText
     * @param feedTypeEnum
     * @return
     */
    private CallApiTrace censorRequest(String eventType, String eventValue, String uniqueID, boolean isText, FeedTypeEnum feedTypeEnum) {
        String dataJSON = censorBuildJSONParms(eventValue, eventType, uniqueID, isText);
        String url = isText ? config.texthost() : config.imghost();

        CallApiTrace trace = new CallApiTrace();
        trace.setReqUrl(url);
        trace.setReqData(dataJSON);
        trace.setData(eventValue);

        String rsp = null;
        try {
            rsp = HttpUtil.post(url, dataJSON);
        } catch (IOException e) {
            log.error(LOG_PREFIX + feedTypeEnum.getName(), e);
            trace.setErrMsg(e.getMessage());
        }

        trace.setRspData(rsp);

        return trace;
    }

    /**
     * 审核创建的发送JSON参数
     *
     * @param eventValue
     * @param eventId
     * @param isText
     * @return
     */
    private String censorBuildJSONParms(String eventValue, String eventId, String uniqueID, boolean isText) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("accessKey", config.accessKey());
        jsonObject.put("appId", config.appId());
        jsonObject.put("eventId", eventId);
        jsonObject.put("type", "AD_PORN_POLITICS");

        JSONObject data = new JSONObject();
        data.put(isText ? "text" : "img", eventValue);

        if (StringUtils.isBlank(uniqueID)) {
            uniqueID = Long.toString(System.currentTimeMillis());
        }
        data.put("tokenId", uniqueID);

        jsonObject.put("data", data);
        return jsonObject.toJSONString();
    }

    private GreenRobotAuditResult getResult(String rsp) {
        GreenRobotAuditResult result = new GreenRobotAuditResult();
        JSONObject json = JSON.parseObject(rsp);
        log.info("GreenRobotAuditResult:" + rsp);
        if (json == null) {
            result.setRobotAuditStatus(GreenAuditStatusEnum.BLOCK_RSP_ERROR);
            result.setFailMsg("返回结果不是JSON格式 " + rsp);
            return result;
        }

        int code = json.getInteger("code").intValue();

        if (code != 1100) {
            //请求失败
            result.setRobotAuditStatus(GreenAuditStatusEnum.BLOCK_RSP_ERROR);
            result.setFailMsg(json.getString("message"));
            return result;
        }

        //审核状态判断
        String statusKye = "riskLevel";
        result.setStatusKye(statusKye);
        String riskLevel = json.getString(statusKye);
        result.setRobotAuditStatus(auditStatusMap.get(riskLevel));
        if (!riskLevel.equals("PASS")) {
            JSONObject data = json.getJSONObject("detail");

            result.setFailMsg(json.getString("detail"));
            // 尝试解析具体错误原因
            if (data != null) {
                String msg = data.getString("description");
                if (StringUtils.isNotBlank(msg)) {
                    result.setFailMsg(msg);
                }
            }
        }
        return result;
    }
}
