package com.starbuds.server.service.data.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.cache.RedisRiskKeys;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.data.GreenCompanyTypeEnum;
import com.starbuds.server.common.pojo.api.CallApiTrace;
import com.starbuds.server.common.pojo.data.RiskParms;
import com.starbuds.server.common.pojo.data.green.GreenRuleDo;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.data.config.ShumeiConfig;
import com.starbuds.server.service.data.dao.DataGreenRuleDAO;
import com.starbuds.server.service.data.entity.green.GreenRobotAuditResult;
import com.starbuds.server.service.data.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

//设备指纹、风控，激活注册登录等业务风控
@Slf4j
@Component(value = "shumeiRiskProvider")
public class ShumeiRiskProvider extends RiskProvider {

    @Resource
    private ShumeiConfig config;

    @Resource
    RedisService redisService;

    @Resource
    DataGreenRuleDAO dataGreenRuleDAO;

    private final String LOG_PREFIX = "数美风控-";

    @Override
    protected CallApiTrace loginCensor(RiskParms parms) {
        return post(buildParms(parms, "login"), GreenBizTypeEnum.USER_LOGIN);
    }

    @Override
    protected CallApiTrace registCensor(RiskParms parms) {
        return post(buildParms(parms, "register"), GreenBizTypeEnum.USER_REGIST);
    }

    @Override
    protected CallApiTrace activationCensor(RiskParms parms) {
        return post(buildParms(parms, "activation"), GreenBizTypeEnum.USER_ACTIVATION);
    }


    private String buildParms(RiskParms parms, String type) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("accessKey", config.accessKey());
        jsonObject.put("appId", config.appId());
        jsonObject.put("eventId", type);

        JSONObject data = new JSONObject();
        if (!type.equals("activation")) {
            data.put("tokenId", parms.getUserID().toString());
            if (parms.getLng() != null && parms.getLat() != null) {
                JSONObject gps = new JSONObject();
                gps.put("gpsLongitude", parms.getLng());
                gps.put("gpslatitude", parms.getLat());
                data.put("gpsLocation", gps);
            }
        }

        if (StringUtils.isNotEmpty(parms.getPhone())) {
            data.put("phone", parms.getPhone());
            data.put("countryCode", parms.getCountryCode());
        }

        data.put("isTokenSeperate", 1);
        data.put("timestamp", parms.getTimestamp());
        data.put("deviceId", parms.getDeviceId());
        data.put("ip", parms.getIp());
        if (parms.getOs().equalsIgnoreCase("h5")) {
            data.put("os", "web");
        } else {
            data.put("os", parms.getOs().toLowerCase());
        }

        data.put("appVersion", parms.getAppVersion());

        if (type.equals("activation")) {
            data.put("advertisingId", parms.getAdvertisingId());
            data.put("apputm", parms.getApputm());
        }

        jsonObject.put("data", data);
        return jsonObject.toJSONString();
    }

    /**
     * 审核状态Map
     */
    private Map<String, GreenAuditStatusEnum> auditStatusMap;

    @PostConstruct
    private void init() {
        // 1.合规，2.不合规，3.疑似，4.审核失败
        auditStatusMap = new HashMap<>();
        auditStatusMap.put("PASS", GreenAuditStatusEnum.PASSED_ROBOT);//通过
        auditStatusMap.put("REVIEW", GreenAuditStatusEnum.PASSED_ROBOT);//需要审核
        auditStatusMap.put("REJECT", GreenAuditStatusEnum.REVIEW_ROBOT);//拒绝
        auditStatusMap.put("VERIFY", GreenAuditStatusEnum.PASSED_ROBOT);//需要验证
    }


    /**
     * 请求数据
     *
     * @param parmsJson
     * @param greenBizTypeEnum
     * @return
     */
    private CallApiTrace post(String parmsJson, GreenBizTypeEnum greenBizTypeEnum) {
        String url = config.riskHost();
        CallApiTrace trace = new CallApiTrace();
        trace.setReqUrl(url);
        trace.setReqData(parmsJson);

        String rsp = null;
        try {
            rsp = HttpUtil.post(url, parmsJson);
        } catch (IOException e) {
            log.error(LOG_PREFIX + greenBizTypeEnum.getValue(), e);
            trace.setErrMsg(e.getMessage());
        }

        trace.setRspData(rsp);
        return trace;
    }


    private GreenRobotAuditResult getResult(String rsp) {
        GreenRobotAuditResult result = new GreenRobotAuditResult();
        JSONObject json = JSON.parseObject(rsp);
        if (json == null) {
            result.setRobotAuditStatus(GreenAuditStatusEnum.BLOCK_RSP_ERROR);
            result.setFailMsg("返回结果不是JSON格式 " + rsp);
            return result;
        }

        int code = json.getInteger("code").intValue();

        if (code != 1100) {
            //请求失败
            result.setRobotAuditStatus(GreenAuditStatusEnum.BLOCK_RSP_ERROR);
            result.setFailMsg(json.getString("message"));
            return result;
        }

        //审核状态判断
        String statusKye = "riskLevel";
        result.setStatusKye(statusKye);
        String riskLevel = json.getString(statusKye);
        result.setRobotAuditStatus(auditStatusMap.get(riskLevel));
        if (!riskLevel.equals("PASS")) {
            //审核拒绝

            JSONObject data = json.getJSONObject("detail");

            result.setFailMsg(json.getString("detail"));
            // 尝试解析具体错误原因
            if (data != null) {
                String msg = data.getString("description");
                if (StringUtils.isNotBlank(msg)) {
                    result.setFailMsg(msg);
                }

                String model = data.getString("model");
                if (StringUtils.isNotBlank(model)) {
                    //后台定义的审核拒绝结果是否启用
                    String modelValue = redisService.get(RedisRiskKeys.getRiskRule(model));
                    if (modelValue == null) {
                        GreenRuleDo greenRuleDo = dataGreenRuleDAO.getRule(model);
                        if (greenRuleDo != null) {
                            modelValue = greenRuleDo.getStatus().toString();
                            redisService.set(RedisRiskKeys.getRiskRule(model), modelValue);
                        } else {
                            //如果后台没有定义规则的话就全部放过
                            result.setRobotAuditStatus(GreenAuditStatusEnum.PASSED_ROBOT);
                        }
                    }

                    if (modelValue != null && modelValue.equals("0")) {
                        //数美拒绝，但后台规则设置为不启用拒绝策略
                        result.setRobotAuditStatus(GreenAuditStatusEnum.PASSED_ROBOT);
                    }
                }

            }
        }
        return result;
    }


    @Override
    protected GreenRobotAuditResult loginResult(String rsp) {
        return getResult(rsp);
    }

    @Override
    protected GreenRobotAuditResult registResult(String rsp) {
        return getResult(rsp);
    }

    @Override
    protected GreenRobotAuditResult activationResult(String rsp) {
        return getResult(rsp);
    }

    @Override
    protected GreenCompanyTypeEnum companyType() {
        return GreenCompanyTypeEnum.SHU_MEI;
    }


}
